package com.dragon.tj.portal.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dragon.tj.portal.auth.model.LoginUser;
import com.dragon.tj.portal.auth.web.service.SysDeptService;
import com.dragon.tj.portal.common.base.R;
import com.dragon.tj.portal.common.constants.BusinessConstants;
import com.dragon.tj.portal.common.convert.mesage.MessageReqConvert;
import com.dragon.tj.portal.common.convert.mesage.MessageInfoVOConvert;
import com.dragon.tj.portal.common.dto.message.MessageInfoReq;
import com.dragon.tj.portal.common.dto.message.MessageInfoSend;
import com.dragon.tj.portal.common.enums.message.MessageInfoErrorEnums;
import com.dragon.tj.portal.common.enums.message.ScopeEnums;
import com.dragon.tj.portal.common.enums.message.ReadStatusEnum;
import com.dragon.tj.portal.common.vo.message.MessageInfoParam;
import com.dragon.tj.portal.common.vo.message.MessageInfoVO;
import com.dragon.tj.portal.component.exception.message.MessageInfoException;
import com.dragon.tj.portal.component.message.KafkaInitialConfiguration;
import com.dragon.tj.portal.component.message.MessageProducer;
import com.dragon.tj.portal.entity.MessageInfo;
import com.dragon.tj.portal.entity.MessageInfoScope;
import com.dragon.tj.portal.mapper.MessageInfoMapper;
import com.dragon.tj.portal.service.MessageInfoScopeService;
import com.dragon.tj.portal.service.MessageInfoService;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Set;

/**
 * <p>
 * 信息表 服务实现类
 * </p>
 *
 * @author huey
 * @since 2023-06-15
 */
@Service
public class MessageInfoServiceImpl extends ServiceImpl<MessageInfoMapper, MessageInfo> implements MessageInfoService {

    @Autowired
    private MessageProducer producer;

    @Autowired
    private MessageInfoScopeService messageInfoScopeService;

    @Autowired
    private SysDeptService sysDeptService;

    @Autowired
    private MessageReqConvert messageReqConvert;

    @Autowired
    private MessageInfoVOConvert messageInfoVOConvert;

    @Override
    public R push(MessageInfoReq messageInfoReq, LoginUser loginUser) {
        return R.ok(this.doMessage(messageInfoReq, loginUser));
    }

    @Override
    public MessageInfoVO detail(Long id) {
        MessageInfo messageInfo = this.getById(id);
        return messageInfoVOConvert.toVo(messageInfo);
    }

    @Override
    public boolean update(MessageInfoParam messageInfoParam) {
        MessageInfo messageInfo = messageInfoVOConvert.paramTo(messageInfoParam);
        return this.updateById(messageInfo);
    }

    @Override
    public boolean read(Long id) {
        MessageInfo messageInfo = this.getById(id);
        Integer readStatus = messageInfo.getReadStatus();
        if (ReadStatusEnum.YES.value().equals(readStatus)) {
            throw new MessageInfoException(MessageInfoErrorEnums.Code.MESSAGE_READ_ALREADY);
        }
        messageInfo.setReadStatus(ReadStatusEnum.YES.value());
        return this.updateById(messageInfo);
    }


    /**
     * @author huey China.
     * @Description :
     * @Date Created in 2023/6/16 11:44
     */
    private boolean transferIds(MessageInfoReq messageInfoReq, LoginUser loginUser) {
        // 120101510000 120000450200
        //120222197001010001 120222197001010002
        boolean isSend = false;
        Integer messageType = messageInfoReq.getMessageType();
        Set<String> scopeIds = messageInfoReq.getScopeIds();
        Set<String> messageClientIds = this.getMessageClientIds(messageType, scopeIds);

        if (CollUtil.isNotEmpty(messageClientIds)) {
            MessageInfoSend messageInfoSend = this.doSaveMessageInfo(messageInfoReq, messageClientIds, loginUser);
            isSend = producer.send(KafkaInitialConfiguration.sseTopic, JSON.toJSONString(messageInfoSend));
        }
        return isSend;
    }

    private MessageInfoSend doSaveMessageInfo(MessageInfoReq messageInfoReq, Set<String> messageClientIds, LoginUser loginUser) {
        Integer messageType = messageInfoReq.getMessageType();
        ScopeEnums scopeEnums = ScopeEnums.ofMessageType(messageType);
        List<MessageInfoScope> insertList = Lists.newArrayList();
        MessageInfoSend messageInfoSend = new MessageInfoSend();
        messageInfoSend.setMessageInfoReq(messageInfoReq);
        messageInfoSend.setClientIds(messageClientIds);
        messageClientIds.forEach(e -> {
            MessageInfoScope messageInfoScope = new MessageInfoScope();
            messageInfoScope.setScopeId(e);
            messageInfoScope.setScopeType(scopeEnums.value());
            //messageInfoScope.setScopeLevel("");
            insertList.add(messageInfoScope);
        });

        MessageInfo messageInfo = messageReqConvert.reqToInfo(messageInfoReq);
        messageInfo.setCreateBy(loginUser.getIdCard());
        messageInfo.setCreateUser(loginUser.getUsername());
        // 存储
        boolean isSaveBatchSuccess = false;
        try {
            isSaveBatchSuccess = this.save(messageInfo);
            isSaveBatchSuccess = messageInfoScopeService.saveBatch(insertList);
        } catch (Exception e) {
            log.warn("消息中心 saveBatch 存储失败 {}");
        }

        // 批量存储失败，则逐个存储
        if (!isSaveBatchSuccess) {
            try {
                this.save(messageInfo);
                insertList.forEach(e -> {
                    messageInfoScopeService.save(e);
                });
            } catch (Exception e) {
                log.warn("消息中心 存储失败 {}");
            }
        }
        return messageInfoSend;
    }

    private Set<String> getMessageClientIds(Integer messageType, Set<String> scopeIds) {
        //登录人标识
        Set<String> messageClientIds = Sets.newHashSet();
        if (CollUtil.isEmpty(scopeIds)) {
            if (messageType.equals(BusinessConstants.DICT_ITEM_ID_2)) {
                throw new MessageInfoException(MessageInfoErrorEnums.Code.MESSAGE_CONTRACTS_EMPTY);
            } else {
                throw new MessageInfoException(MessageInfoErrorEnums.Code.MESSAGE_DEPT_EMPTY);
            }
        }
        //测试联系人1
        if (messageType.equals(BusinessConstants.DICT_ITEM_ID_2) && scopeIds.contains("1")) {
            messageClientIds.add("c1");
            messageClientIds.add("c2");
        } else {
            Set<String> allUserIds = sysDeptService.getAllUserIds(scopeIds, false);
            if (CollUtil.isNotEmpty(allUserIds)) {
                messageClientIds.addAll(allUserIds);
            }
        }
        return messageClientIds;
    }


    /**
     * @author huey China.
     * @Description : 预先处理 TODO message 没走字典
     * @Date Created in 2023/6/16 11:25
     */
    private boolean doMessage(MessageInfoReq messageInfoReq, LoginUser loginUser) {

        this.validate(messageInfoReq);
        return this.transferIds(messageInfoReq, loginUser);
    }

    /**
     * @author huey China.
     * @Description : md5 验证消息唯一性
     * @Date Created in 2023/6/16 16:15
     */
    private void validate(MessageInfoReq messageInfoReq) {
//        String correctHash = DigestUtils.md5Hex("currentId" + messageInfoReq.getT());
//        if (!correctHash.equalsIgnoreCase(messageInfoReq.getK())) {
//            return;
//        }
    }

}
